@@ -25,24 +25,6 @@ DEFAULT_CHUNK_SIZE = 1024 * 4
 DEFAULT_FORMAT = 'wav'
 
 
-def _prepare_synthesis_params(old: dict, data: dict):
-    def _set():
-        if key in old and old[key] != val:
-            old[key] = val
-            return True
-        return False
-
-    adv = {'punctuation_mode': 3, 'capitals_mode': 4, 'flags': 1}
-    change = False
-    for key, val in data.items():
-        if key in adv:
-            if isinstance(val, int) and 0 <= val <= adv[key]:
-                change |= _set()
-        elif isinstance(val, (int, float)) and -2 <= val <= 2.5:
-            change |= _set()
-    return change
-
-
 class _WaveWrite(wave.Wave_write):
     def _ensure_header_written(self, _):
         pass
@@ -193,44 +175,27 @@ class _AudioWorker:
 class _BaseTTS:
     RELEASE_TIMEOUT = 3
 
-    def __init__(self, pipe: _StreamPipe, free, cmd: dict, allow_formats: frozenset, **kwargs):
+    def __init__(self, is_multiprocessing: bool, free, cmd: dict, allow_formats: frozenset, **kwargs):
+        _event = multiprocessing.Event if is_multiprocessing else threading.Event
         self._free = free
         self._allow_formats = allow_formats
         self._kwargs = kwargs.copy()
         self._lib_path = {} if 'lib_path' not in self._kwargs else {'lib_path': self._kwargs.pop('lib_path')}
-        self._wait = threading.Event()
-        self._pipe = _Pipe()
+        self._wait = _event()
+        self._pipe = _Pipe(is_multiprocessing=is_multiprocessing)
         self._format = DEFAULT_FORMAT
         self._chunk_size = DEFAULT_CHUNK_SIZE
         self._engine = None
-        self._synthesis_param = None
-        self._worker = _AudioWorker(cmd=cmd, pipe=pipe)
+        self._worker = _AudioWorker(cmd=cmd, pipe=_StreamPipe(is_multiprocessing=is_multiprocessing))
         self._work = True
-        self._client_here = threading.Event()
+        self._client_here = _event()
         self._client_here.set()
-        self._generator_work = threading.Event()
+        self._generator_work = _event()
         self._generator_work.set()
         self._still_processing = False
-        self._rollback_sets = None
-
-    def _change_sets(self, sets: dict or None):
-        if not sets:
-            return
-        old_params = self._synthesis_param.copy()
-        if _prepare_synthesis_params(self._synthesis_param, sets):
-            self._engine.set_params(**self._synthesis_param)
-            if self._rollback_sets is None:
-                self._rollback_sets = old_params
-
-    def _restore_sets(self):
-        if self._rollback_sets:
-            self._synthesis_param = self._rollback_sets
-            self._engine.set_params(**self._synthesis_param)
-            self._rollback_sets = None
 
     def _engine_init(self):
         self._engine = rhvoice_proxy.Engine(**self._lib_path)
-        self._synthesis_param = self._engine.synthesis_set.copy()  # Current params
         self._engine.init(self._speech_callback, self._sr_callback, **self._kwargs)
 
     def _engine_destroy(self):
@@ -269,9 +234,12 @@ class _BaseTTS:
     def _client_request(self, text, voice, format_, chunk_size, sets):
         if format_ not in self._allow_formats:
             raise RuntimeError('Unsupported format: {}'.format(format_))
-        if sets is not None and not isinstance(sets, dict):
+        sets = sets or {}
+        if not isinstance(sets, dict):
             RuntimeError('Sets must be dict or None')
-        self._pipe.put((text, voice, format_, chunk_size, sets))
+        if voice:
+            sets['voice_profile'] = voice
+        self._pipe.put((text, format_, chunk_size, sets))
         self._wait.wait(3600)
         self._wait.clear()
 
@@ -319,25 +287,29 @@ class _BaseTTS:
         if buffer:
             yield buffer
 
-    def _generate(self, text, voice, format_, chunk_size, sets):
+    def _get_temporary_params(self, sets):
+        try:
+            return self._engine.params.copy_with(sets)
+        except Exception as e:
+            print('sets error: {}'.format(e))
+        return None
+
+    def _generate(self, text, format_, chunk_size, sets):
         self._generator_work.clear()
-        self._change_sets(sets)
         self._format = format_
         self._chunk_size = chunk_size or DEFAULT_CHUNK_SIZE
-        if voice:
-            self._engine.set_voice(voice)
+        params = self._get_temporary_params(sets) if sets else None
         try:
             if isinstance(text, str):
-                self._engine.generate(text)
+                self._engine.generate(text=text, params=params)
             elif isinstance(text, Iterable):
                 for chunk in text:
-                    self._engine.generate(chunk)
+                    self._engine.generate(chunk, params=params)
         except RuntimeError:
             pass
         self._still_processing = False
         if not self._worker.end_processing():
             self._wait.set()
-        self._restore_sets()
         self._release_busy()
 
     def run(self):
@@ -347,40 +319,36 @@ class _BaseTTS:
             if data is None:
                 break
             if isinstance(data, dict):
-                self._synthesis_param = data
                 self._engine.set_params(**data)
             else:
                 self._generate(*data)
         self._engine_destroy()
 
+    def stop(self):
+        if self._work:
+            self._work = False
+            self._pipe.put(None)
+
 
 class ThreadTTS(_BaseTTS, threading.Thread):
     def __init__(self, *args, **kwargs):
         threading.Thread.__init__(self)
-        _BaseTTS.__init__(self, _StreamPipe(), *args, **kwargs)
+        _BaseTTS.__init__(self, False, *args, **kwargs)
         self.start()
 
     def join(self, timeout=None):
-        self._work = False
-        self._pipe.put(None)
+        self.stop()
         super().join()
 
 
 class ProcessTTS(_BaseTTS, multiprocessing.Process):
     def __init__(self, *args, **kwargs):
         multiprocessing.Process.__init__(self)
-        _BaseTTS.__init__(self, _StreamPipe(True), *args, **kwargs)
-        self._wait = multiprocessing.Event()
-        self._pipe = _Pipe(True)
-        self._client_here = multiprocessing.Event()
-        self._client_here.set()
-        self._generator_work = multiprocessing.Event()
-        self._generator_work.set()
+        _BaseTTS.__init__(self, True, *args, **kwargs)
         self.start()
 
     def join(self, timeout=None):
-        self._work = False
-        self._pipe.put(None)
+        self.stop()
         super().join()
 
 
@@ -398,15 +366,14 @@ class MultiTTS:
         self._lock = threading.Lock()
         self._work = True
 
-    def to_file(self, filename: str, text: str, voice='anna', format_=DEFAULT_FORMAT, sets=None):
+    def to_file(self, filename: str, text: str, voice=None, format_=None, sets=None):
         return self._caller().to_file(filename, text, voice, format_, sets)
 
-    def say(self, text: str, voice='anna', format_=DEFAULT_FORMAT, buff=DEFAULT_CHUNK_SIZE, sets=None):
+    def say(self, text: str, voice=None, format_=None, buff=DEFAULT_CHUNK_SIZE, sets=None):
         return self._caller().say(text, voice, format_, buff, sets)
 
     def _caller(self):
-        self._lock.acquire()
-        try:
+        with self._lock:
             while True:
                 self._free.clear()
                 for worker in self._workers:
@@ -415,8 +382,6 @@ class MultiTTS:
                         return worker
                 if not self._free.wait(self.TIMEOUT):
                     raise RuntimeError('Still busy')
-        finally:
-            self._lock.release()
 
     def set_params(self, **kwargs):
         for worker in self._workers:
@@ -426,6 +391,7 @@ class MultiTTS:
         if not self._work:
             return
         self._work = False
+        [x.stop() for x in self._workers]
         [x.join() for x in self._workers]
 
 
@@ -440,12 +406,13 @@ class TTS:
         'opus_path': 'OPUSENCPATH',
         'flac_path': 'FLACPATH',
         'quiet': 'QUIET',
+        'config_path': 'RHVOICECONFIGPATH',
     }
 
     def __init__(self, threads=_unset, force_process=_unset,
                  lib_path=_unset, data_path=_unset, resources=_unset,
                  lame_path=_unset, opus_path=_unset, flac_path=_unset,
-                 quiet=_unset
+                 quiet=_unset, config_path=_unset,
                  ):
         """
         :param int or bool or None threads: If equal to 1, created one thread object,
@@ -462,6 +429,8 @@ class TTS:
         :param str or None opus_path: Path to opusenc, optional. File must be present for opus support. Default opusenc.
         :param str or None flac_path: Path to flac, optional. File must be present for flac support. Default flac.
         :param bool or None quiet: If True don't info output. Default False.
+        :param str or None config_path: Path to folder, contain RHVoice.conf in linux and RHVoice.ini in windows.
+        Default /usr/local/etc/RHVoice.
         """
         kwargs = {}
         for key in self.PARAMS:
@@ -494,7 +463,7 @@ class TTS:
             )
         test.init(play_speech_cb=lambda *_: True, set_sample_rate_cb=lambda *_: True, **envs2)
         self._voices = test.voices
-        self._synth_set = test.synthesis_set.copy()
+        self._params = test.params
         del test
 
         tts = MultiTTS(self._threads, self._process, self._cmd, self._formats, **envs)
@@ -537,16 +506,19 @@ class TTS:
         return self._cmd
 
     def set_params(self, **kwargs):
-        if _prepare_synthesis_params(self._synth_set, kwargs):
-            self.__set_params(**self._synth_set)
-            return True
-        else:
-            return False
+        result = False
+        try:
+            result = self._params.update_from_dict(kwargs)
+        except RuntimeError as e:
+            print('set_params error: {}'.format(e))
+        if result:
+            self.__set_params(**kwargs)
+        return result
 
     def get_params(self, param=None):
         if param is None:
-            return self._synth_set.copy()
-        return self._synth_set.get(param)
+            return self._params.to_dict()
+        return self._params.get_param(param)
 
     def _get_environs(self, kwargs):
         result = {}
